"""Provide meta-models for Asset Administration Shell information model."""

__version__ = "2021.11.20a2"
__author__ = (
    "Nico Braunisch, Marko Ristin, Robert Lehmann, Marcin Sadurski, Manuel Sauer"
)
__license__ = "License :: OSI Approved :: MIT License"
__status__ = "Alpha"
